{% extends 'webgateway/core_html.html' %}
{% load i18n %}

{% comment %}
<!--
  Copyright (C) 2011 University of Dundee & Open Microscopy Environment.
  All rights reserved.

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->
{% endcomment %}


{% block link %}
    {{ block.super }}
    <link rel="stylesheet" href="{% static "webgateway/css/ome.header.css"|add:url_suffix %}" type="text/css" />
    <link rel="stylesheet" href="{% static "webclient/css/layout.css"|add:url_suffix %}" type="text/css" media="screen"/>
{% endblock %}


{% block script %}
    {{ block.super }}
    {% include "webgateway/base/includes/script_src_jquery.html" %}

	<!--[if lte IE 8]>
	<script>
		$(function(){
		   $(".header_toolbar li:last-child").addClass('last-child'),
		   $("#activities_panel table tr:last-child").addClass('last-child'),
		    $("#groupsUsersButton span:last-child").addClass('last-child')
		   
		})
		</script>
	<![endif]-->
	
{% endblock %}


{% block body %}
        
        {% block middle_header %}
        <div id="middle_header">

            <div id="middle_header_logo">
				{% block middle_header_logo %}
				    {% include "webclient/base/includes/logo.html" %}
				{% endblock %}
            </div>


            <div id="middle_header_left">
				{% block middle_header_left %}
				    {% include "webclient/base/includes/menu.html" %}
				{% endblock %}
            </div>
			
			
            <div id="middle_header_right">
				{% block middle_header_right %}
				{% endblock %}
            </div>
			
	        <div class="clear"></div>
	
		</div>
    	{% endblock %}
        
        <div id="content">
        {% block content %}
        {% endblock %}
        </div>
		
		<!--
        {% block footer %}
            <div id="footer">
                {% block copywrite %}
                    <p>&copy; 2007-2011 University of Dundee &amp; Open Microscopy Environment</p>
                {% endblock %}
            </div>
        {% endblock %}
		-->

{% endblock %}