{% comment %}
<!--
  Copyright (C) 2012 University of Dundee & Open Microscopy Environment.
  All rights reserved.

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->
{% endcomment %}

<script>

$(document).ready(function() {
    
    $("#metadata_tab").omeroweb_right_plugin({
        plugin_index:{{ forloop.counter }},
        load_plugin_content: function(selected, obj_dtype, obj_id) {    // Url based on selected object(s)
            var well_index = selected[0]["index"] || 0;
            var url;
            if (selected[0]["shareId"]) {
                url = '{% url 'webindex' %}metadata_acquisition/'+obj_dtype+'/'+obj_id+'/'+selected[0]["shareId"]+'/';
            } else {
                url = '{% url 'webindex' %}metadata_acquisition/'+obj_dtype+'/'+obj_id+'/';
            }
            if (well_index > 0) {
                url = url + '?index='+well_index;
            }
            $(this).html("<h1 style='padding: 20px 15px'><img src='{% static 'webgateway/img/spinner.gif' %}'/>Loading metadata...</h1>");
            $(this).load(url);
        },
        supported_obj_types: ['image']
    });

});

</script>