{% extends "base_error.html" %}
{% load i18n %}

{% comment %}
<!--
  Copyright (C) 2014 University of Dundee & Open Microscopy Environment.
  All rights reserved.

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->
{% endcomment %}

{% block title %}
    {% trans "OMERO.web - Help" %}
{% endblock %}

{% block body %}
    <div class="bodyWrapper">
        <h1>{% trans "OMERO.web Help" %}</h1>

        
        <p>
            {% trans "You may find the following resources useful. If you have any further questions, comments or suggestions, please use the feedback form below." %}
        
        <ul style="list-style: disc inside none">
            <li>
                <a href="https://help.openmicroscopy.org/">User Assistance</a>:
                More information about the OMERO clients.
            </li>
            <li>
                <a href="https://www.openmicroscopy.org">Open Microscopy Environment</a>:
                Home page of the OME project.
            </li>
            <li>
                <a href="https://www.openmicroscopy.org/omero">About OMERO</a>:
                Background and introduction to OMERO.
            </li>
        </ul>
        </p>

        <hr />

        <h1>{% trans "Send Feedback" %}</h1>
        
        <p>{% trans "OME's QA system records any problem or comment you have submitted, your email address, and the IP address you are currently using. We store this information in a password-protected system. If you'd prefer us not to hold this information, please don't submit any information to OME QA. Any feedback you provide will be used to further the development of the OMERO.web and improve our software." %}</p>

        {% if error %}<ul class="errorlist"><li>{{ error }}</li></ul>{% endif %}
        
        <form action="{% url 'csend' %}" method="post" class="standard_form">{% csrf_token %}
            {% for field in form %}
                {% if field.field.required %}<span class="required">{{ field.label_tag }}</span>{% else %}{{ field.label_tag }}{% endif %}
                {{ field }}
                {% if field.errors %}{{ field.errors }}{% endif %}
                {{ field.help_text|safe }}
                <br/>
            {% endfor %}
                <input type="submit" value="Send" />
        </form>
    </div>
{% endblock %}