/*
 * Copyright (C) 2012 University of Dundee & Open Microscopy Environment.
 * All rights reserved.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
 


 
 .left_panel_inner > table { 
	 -webkit-box-shadow: 0 1px 1px rgba(0,0,0,0.1); /* Saf3.0+, Chrome */
	    -moz-box-shadow: 0 1px 1px rgba(0,0,0,0.1); /* FF3.5+ */
	         box-shadow: 0 1px 1px rgba(0,0,0,0.1); /* Opera 10.5, IE 9.0 */
 }

    

    .cal_nav input {
        background-color:rgba(0,0,0,.2);
        text-shadow:0 -1px 0 rgba(0,0,0,.6);
		
    }
    
    .cal_nav input:hover {
        background:rgba(255,255,255,.2);
    }
    
    .cal_nav input:active, .cal_nav input:focus {
        background-color:rgba(0,0,0,.2);
        -webkit-box-shadow: inset 0 1px 2px rgba(0,0,0,.4), 0 1px 0 rgba(255,255,255,.05);
		-webkit-box-shadow:inset 0 1px 2px rgba(0,0,0,0.4), 0 1px 0 rgba(255,255,255,.05); /* Saf3.0+, Chrome */
		   -moz-box-shadow:inset 0 1px 2px rgba(0,0,0,0.4), 0 1px 0 rgba(255,255,255,.05); /* FF3.5+ */
		        box-shadow:inset 0 1px 2px rgba(0,0,0,0.4), 0 1px 0 rgba(255,255,255,.05); /* Opera 10.5, IE 9.0 */ 
    }
    
    

	#prevMonth {
		border-right:solid 1px rgba(0,0,0,.1); 
		
		-webkit-box-shadow: 1px 0 0 rgba(255,255,255,.2); /* Saf3.0+, Chrome */
		   -moz-box-shadow: 1px 0 0 rgba(255,255,255,.2); /* FF3.5+ */
		        box-shadow: 1px 0 0 rgba(255,255,255,.2); /* Opera 10.5, IE 9.0 */
	}
	
	#nextMonth {
		border-left:solid 1px rgba(0,0,0,.1);
		
		-webkit-box-shadow:inset 1px 0 0 rgba(255,255,255,0.2); /* Saf3.0+, Chrome */
		   -moz-box-shadow:inset 1px 0 0 rgba(255,255,255,0.2); /* FF3.5+ */
		        box-shadow:inset 1px 0 0 rgba(255,255,255,0.2); /* Opera 10.5, IE 9.0 */
	}
    
    
/* Actual table
 */ 

	.left_panel_inner > table {
		-webkit-border-radius:4px; /* Saf3+, Chrome */
		   -moz-border-radius:4px; /* FF1+ */
		        border-radius:4px; /* Opera 10.5, IE 9 */ 
	}



/* Header - Days */    
    
.calendar thead {
	background-color:#EEF0F1;	
	filter:progid:DXImageTransform.Microsoft.gradient( startColorstr='rgba(255,255,255,.4)', endColorstr='rgba(255,255,255,0)',GradientType=0 ); /* IE6-9 */
}    
    
    .calendar thead th {
        border-right: solid 1px rgba(0,0,0,.05);
        text-shadow:0 1px 0 rgba(255,255,255,1);
		border-bottom:solid 1px #D4D6D8;
		color: #737F8C;
    }
    



/* Header - Top Days */



    tbody .calendar_day {
        border-left:solid 1px rgba(0,0,0,.02);
        border-bottom:solid 1px rgba(0,0,0,.02);
    }
    
    tbody .not_calendar_day {
        border-left:solid 1px rgba(0,0,0,.01);
        border-bottom:solid 1px rgba(0,0,0,.01);
    }
    
        .calendar_day a:hover {
             background-color: #EEF4F6;
        }
    




/* Middle - Day Numbers */

.calendar_numerical_day {
    color:rgba(0,0,0,.7);
    text-shadow:0 1px 0 white;    
}

















