
<div class="split_filesets_info">
    <h1>
        <img style="vertical-align: middle; position:relative; top:-3px" src="{% static "webgateway/img/failed.png" %}" />
        Multi-image filesets cannot be 
        {% ifequal action 'move' %}
            split into 2 groups.
        {% else %}
            partially deleted.
        {% endifequal %}
    </h1>

    {% for fs in split_filesets %}
    <div class="split_fileset">
        <div>
            You tried to {{ action }} <strong>{{ fs.attempted_iids|length }}</strong>
            image{{ fs.attempted_iids|pluralize}} from a fileset,
            leaving <strong>{{ fs.blocking_iids|length }}</strong>.
            <!-- When we add this html to chgrp form - fileset will be included -->
            <input name="fileset" value="{{ fs.id }}" style="display:none"/>
        </div>
        {% with MAX_SHOWN=4 %}
        <div style="border: 1px solid #87ABD2; float: left">
            {% for iid in fs.attempted_iids|slice:":4" %}
                <a href="{% url 'webindex' %}?show=image-{{ iid }}">
                    <img src="{% url 'render_thumbnail_resize' 96 iid %}" class="fileset_image"/>
                </a>
            {% endfor %}
            {% if fs.attempted_iids|length > MAX_SHOWN %}
                <div style="height:35px; float: left; line-height:65px"> ... </div>
            {% endif %}
        </div>
        <div style="border: 1px solid #87ABD2; float: left; margin-left:35px">
            {% for iid in fs.blocking_iids|slice:":4" %}
                <a href="{% url 'webindex' %}?show=image-{{ iid }}">
                    <img src="{% url 'render_thumbnail_resize' 96 iid %}" class="fileset_image"/>
                </a>
            {% endfor %}
            {% if fs.blocking_iids|length > MAX_SHOWN %}
                <div style="height:35px; float: left; line-height:65px"> ... </div>
            {% endif %}
        </div>
        <div style="clear:both"></div>
    {% endwith %}

    </div>
    {% endfor %}

</div>