{% extends "webclient/login.html" %}
{% load i18n %}

{% comment %}
<!--
  Copyright (C) 2011-2013 University of Dundee & Open Microscopy Environment.
  All rights reserved.

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->
{% endcomment %}

{% block title %}
  OMERO.web
{% endblock %}

{% block login %}

    {% if error %}<h1 class="error">Following error occurred: INDEX_TEMPLATE, '{{ error }}'</h1>{% endif %}

    <form class="standard_form inlined">
      <h1 style="text-align: center">This is index template example.</h1>
      <p id="goWebclientMsg" style="text-align: center; margin-top: 15px; margin-bottom:0px; position: relative; top:15px">
        To <span>go</span> to the webclient, click <a href="{% url 'webindex' %}">here</a>
      </p>
    </form>

{% endblock %}








