{% load i18n %}


{% comment %}
<!--
  Copyright (C) 2011 University of Dundee & Open Microscopy Environment.
  All rights reserved.

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->
{% endcomment %}

<h2>{% trans "Most recent tags" %}:</h2>
{% if controller.mostRecentTags %}
    <div class="tags">
            {% for key,value in controller.mostRecentTags.items %}
              
				<span class="tag_annotation_wrapper">
					<div class="tag">
						<a class="tag_inner" href="{% url 'load_template' "usertags" %}{% if not value.obj.isOwned %}?experimenter={{  value.obj.getOwner.id }}{% endif %}" class="tag" target="_top">{{ value.obj.textValue|escape }}</a>
					</div>
				</span>	
           
		    {% endfor %}
    </div>
{% else %}
    <div><p>{% trans "No tags." %}</p></div>
{% endif %}
