{% comment %}
<!--
  Copyright (C) 2011 University of Dundee & Open Microscopy Environment.
  All rights reserved.

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->
{% endcomment %}

{% if ome.user_dropdown %}
{% if ome.is_public_user %}
    <div id="public_login_button">
        <a href="{% url 'weblogin' %}">
            <div id="show_public_login_button">
                <img src="{% static 'webclient/image/personal32.png' %}"/>
                <span>Login</span>
            </div>
        </a>
    </div>
{% else %}
    <div id="user_dropdown">
        <div id="show_user_dropdown">
            <img src="{% url 'wamyphoto' %}" />
            <span>{{ ome.user.getFullName }}</span>
        </div>
        <ul class="dropdown">
            {% if ome.can_create %}
              <li id="user_settings"><a href="{% url 'wamyaccount' 'edit' %}" title="User settings">
                User settings
              </a></li>
            {% endif %}
			<!-- Help -->
            <li id="send"><a href="#" onclick="return OME.openPopup('{% url 'csend' %}')">Send Feedback</a></li>
			<li id="logout">{% include "webclient/base/includes/logout.html" %}</li>
        </ul>
    </div>
{% endif %}{# ome.is_public_user #}
{% endif %}{# ome.user_dropdown #}
